@@ -7,23 +7,7 @@
  ******************************************************************************
  * Copyright (c) 2023, Even Rouault <even dot rouault at spatialys dot com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_port.h"
@@ -50,9 +34,10 @@
 #include "gdal_priv.h"
 #include "ogr_api.h"
 #include "ogr_core.h"
-#include "ogr_mem.h"
+#include "memdataset.h"
 #include "ogrsf_frmts.h"
 #include "ogr_spatialref.h"
+#include "gdalargumentparser.h"
 
 constexpr const char *DEFAULT_LAYER_NAME = "footprint";
 
@@ -116,12 +101,176 @@ struct GDALFootprintOptions
     /*! Field name where to write the path of the raster. Empty if not desired */
     std::string osLocationFieldName = "location";
 
+    /*! Clears the osLocationFieldName var when set */
+    bool bClearLocation = false;
+
     /*! Whether to force writing absolute paths in location field. */
     bool bAbsolutePath = false;
 
-    std::string osSrcNoData;
+    std::string osSrcNoData{};
 };
 
+static std::unique_ptr<GDALArgumentParser> GDALFootprintAppOptionsGetParser(
+    GDALFootprintOptions *psOptions,
+    GDALFootprintOptionsForBinary *psOptionsForBinary)
+{
+    auto argParser = std::make_unique<GDALArgumentParser>(
+        "gdal_footprint", /* bForBinary=*/psOptionsForBinary != nullptr);
+
+    argParser->add_description(_("Compute footprint of a raster."));
+
+    argParser->add_epilog(_("For more details, consult "
+                            "https://gdal.org/programs/gdal_footprint.html"));
+
+    argParser->add_argument("-b")
+        .metavar("<band>")
+        .scan<'i', int>()
+        .append()
+        .store_into(psOptions->anBands)
+        .help(_("Band(s) of interest."));
+
+    argParser->add_argument("-combine_bands")
+        .choices("union", "intersection")
+        .action([psOptions](const auto &s)
+                { psOptions->bCombineBandsUnion = s == "union"; })
+        .default_value("union")
+        .help(_("Defines how the mask bands of the selected bands are combined "
+                "to generate a single mask band, before being vectorized."));
+
+    {
+        auto &group = argParser->add_mutually_exclusive_group();
+
+        group.add_argument("-ovr")
+            .metavar("<index>")
+            .scan<'i', int>()
+            .store_into(psOptions->nOvrIndex)
+            .help(_("Defines which overview level of source file must be used, "
+                    "when overviews are available on the source raster."));
+
+        group.add_argument("-srcnodata")
+            .metavar("\"<value>[ <value>]...\"")
+            .store_into(psOptions->osSrcNoData)
+            .help(_("Set nodata value(s) for input bands."));
+    }
+
+    argParser->add_argument("-t_cs")
+        .choices("pixel", "georef")
+        .default_value("georef")
+        .action(
+            [psOptions](const auto &s)
+            {
+                const bool georefSet{s == "georef"};
+                psOptions->bOutCSGeoref = georefSet;
+                psOptions->bOutCSGeorefRequested = georefSet;
+            })
+        .help(_("Target coordinate system."));
+
+    // Note: no store_into (requires post validation)
+    argParser->add_argument("-t_srs")
+        .metavar("<srs_def>")
+        .help(_("Target CRS of the output file.."));
+
+    argParser->add_argument("-split_polys")
+        .flag()
+        .store_into(psOptions->bSplitPolys)
+        .help(_("Split multipolygons into several features each one with a "
+                "single polygon."));
+
+    argParser->add_argument("-convex_hull")
+        .flag()
+        .store_into(psOptions->bConvexHull)
+        .help(_("Compute the convex hull of the (multi)polygons."));
+
+    argParser->add_argument("-densify")
+        .metavar("<value>")
+        .scan<'g', double>()
+        .store_into(psOptions->dfDensifyDistance)
+        .help(_("The specified value of this option is the maximum distance "
+                "between 2 consecutive points of the output geometry. "));
+
+    argParser->add_argument("-simplify")
+        .metavar("<value>")
+        .scan<'g', double>()
+        .store_into(psOptions->dfSimplifyTolerance)
+        .help(_("The specified value of this option is the tolerance used to "
+                "merge consecutive points of the output geometry."));
+
+    argParser->add_argument("-min_ring_area")
+        .metavar("<value>")
+        .scan<'g', double>()
+        .store_into(psOptions->dfMinRingArea)
+        .help(_("The specified value of this option is the minimum area of a "
+                "ring to be considered."));
+
+    // Note: no store_into (requires post validation)
+    argParser->add_argument("-max_points")
+        .metavar("<value>|unlimited")
+        .default_value("100")
+        .help(_("The maximum number of points in the output geometry."));
+
+    if (psOptionsForBinary)
+    {
+        argParser->add_quiet_argument(&psOptionsForBinary->bQuiet);
+        argParser->add_open_options_argument(
+            psOptionsForBinary->aosOpenOptions);
+    }
+
+    argParser->add_output_format_argument(psOptions->osFormat);
+
+    {
+        auto &group = argParser->add_mutually_exclusive_group();
+
+        group.add_argument("-location_field_name")
+            .metavar("<field_name>")
+            .default_value("location")
+            .store_into(psOptions->osLocationFieldName)
+            .help(_(
+                "Specifies the name of the field in the resulting vector "
+                "dataset where the path of the input dataset will be stored."));
+
+        group.add_argument("-no_location")
+            .flag()
+            .store_into(psOptions->bClearLocation)
+            .help(
+                _("Turns off the writing of the path of the input dataset as a "
+                  "field in the output vector dataset."));
+    }
+
+    argParser->add_argument("-write_absolute_path")
+        .flag()
+        .store_into(psOptions->bAbsolutePath)
+        .help(_("Enables writing the absolute path of the input dataset."));
+
+    argParser->add_layer_creation_options_argument(psOptions->aosLCO);
+
+    argParser->add_dataset_creation_options_argument(psOptions->aosDSCO);
+
+    argParser->add_argument("-lyr_name")
+        .metavar("<value>")
+        .store_into(psOptions->osDestLayerName)
+        .help(_("Name of the target layer."));
+
+    if (psOptionsForBinary)
+    {
+        argParser->add_argument("-overwrite")
+            .flag()
+            .store_into(psOptionsForBinary->bOverwrite)
+            .help(_("Overwrite the target layer if it exists."));
+
+        argParser->add_argument("src_filename")
+            .metavar("<src_filename>")
+            .store_into(psOptionsForBinary->osSource)
+            .help(_("Source raster file name."));
+
+        argParser->add_argument("dst_filename")
+            .metavar("<dst_filename>")
+            .store_into(psOptionsForBinary->osDest)
+            .help(_("Destination vector file name."));
+    }
+
+    return argParser;
+}
+
 /************************************************************************/
 /*                       GDALFootprintMaskBand                          */
 /************************************************************************/
@@ -130,6 +279,8 @@ class GDALFootprintMaskBand final : public GDALRasterBand
 {
     GDALRasterBand *m_poSrcBand = nullptr;
 
+    CPL_DISALLOW_COPY_ASSIGN(GDALFootprintMaskBand)
+
   public:
     explicit GDALFootprintMaskBand(GDALRasterBand *poSrcBand)
         : m_poSrcBand(poSrcBand)
@@ -417,7 +568,8 @@ GetOutputLayerAndUpdateDstDS(const char *pszDest, GDALDatasetH &hDstDS,
                 {
                     CPLError(CE_Warning, CPLE_AppDefined,
                              "Several drivers matching %s extension. Using %s",
-                             CPLGetExtension(pszDest), aoDrivers[0].c_str());
+                             CPLGetExtensionSafe(pszDest).c_str(),
+                             aoDrivers[0].c_str());
                 }
                 osFormat = aoDrivers[0];
             }
@@ -455,25 +607,31 @@ GetOutputLayerAndUpdateDstDS(const char *pszDest, GDALDatasetH &hDstDS,
     /* -------------------------------------------------------------------- */
     auto poDstDS = GDALDataset::FromHandle(hDstDS);
     OGRLayer *poLayer = nullptr;
-    if (!psOptions->osDestLayerName.empty())
+
+    if (!bCreateOutput)
     {
-        poLayer = poDstDS->GetLayerByName(psOptions->osDestLayerName.c_str());
-        if (!poLayer)
+        if (poDstDS->GetLayerCount() == 1 && poDstDS->GetDriver() &&
+            EQUAL(poDstDS->GetDriver()->GetDescription(), "ESRI Shapefile"))
         {
-            CPLError(CE_Failure, CPLE_AppDefined, "Cannot find layer %s",
-                     psOptions->osDestLayerName.c_str());
-            return nullptr;
+            poLayer = poDstDS->GetLayer(0);
+        }
+        else if (!psOptions->osDestLayerName.empty())
+        {
+            poLayer =
+                poDstDS->GetLayerByName(psOptions->osDestLayerName.c_str());
+            if (!poLayer)
+            {
+                CPLError(CE_Failure, CPLE_AppDefined, "Cannot find layer %s",
+                         psOptions->osDestLayerName.c_str());
+                return nullptr;
+            }
+        }
+        else
+        {
+            poLayer = poDstDS->GetLayerByName(DEFAULT_LAYER_NAME);
         }
     }
-    else if (poDstDS->GetLayerCount() == 1 && poDstDS->GetDriver() &&
-             EQUAL(poDstDS->GetDriver()->GetDescription(), "ESRI Shapefile"))
-    {
-        poLayer = poDstDS->GetLayer(0);
-    }
-    else
-    {
-        poLayer = poDstDS->GetLayerByName(DEFAULT_LAYER_NAME);
-    }
+
     if (!poLayer)
     {
         std::string osDestLayerName = psOptions->osDestLayerName;
@@ -482,7 +640,7 @@ GetOutputLayerAndUpdateDstDS(const char *pszDest, GDALDatasetH &hDstDS,
             if (poDstDS->GetDriver() &&
                 EQUAL(poDstDS->GetDriver()->GetDescription(), "ESRI Shapefile"))
             {
-                osDestLayerName = CPLGetBasename(pszDest);
+                osDestLayerName = CPLGetBasenameSafe(pszDest);
             }
             else
             {
@@ -877,7 +1035,7 @@ static bool GDALFootprintProcess(GDALDataset *poSrcDS, OGRLayer *poDstLayer,
             CPLAssert(poGeom);
             if (poGeom->getGeometryType() == wkbPolygon)
             {
-                poMP->addGeometryDirectly(poGeom.release());
+                poMP->addGeometry(std::move(poGeom));
             }
         }
         poMemLayer = std::make_unique<OGRMemLayer>("", nullptr, wkbUnknown);
@@ -944,7 +1102,7 @@ static bool GDALFootprintProcess(GDALDataset *poSrcDS, OGRLayer *poDstLayer,
                         }
                     }
                     if (!poNewPoly->IsEmpty())
-                        poMP->addGeometryDirectly(poNewPoly.release());
+                        poMP->addGeometry(std::move(poNewPoly));
                 }
                 poGeom = std::move(poMP);
             }
@@ -1036,8 +1194,8 @@ static bool GDALFootprintProcess(GDALDataset *poSrcDS, OGRLayer *poDstLayer,
                 char *pszCurDir = CPLGetCurrentDir();
                 if (pszCurDir)
                 {
-                    osFilename = CPLProjectRelativeFilename(pszCurDir,
-                                                            osFilename.c_str());
+                    osFilename = CPLProjectRelativeFilenameSafe(
+                        pszCurDir, osFilename.c_str());
                     CPLFree(pszCurDir);
                 }
             }
@@ -1054,6 +1212,28 @@ static bool GDALFootprintProcess(GDALDataset *poSrcDS, OGRLayer *poDstLayer,
     return true;
 }
 
+/************************************************************************/
+/*                  GDALFootprintAppGetParserUsage()                    */
+/************************************************************************/
+
+std::string GDALFootprintAppGetParserUsage()
+{
+    try
+    {
+        GDALFootprintOptions sOptions;
+        GDALFootprintOptionsForBinary sOptionsForBinary;
+        auto argParser =
+            GDALFootprintAppOptionsGetParser(&sOptions, &sOptionsForBinary);
+        return argParser->usage();
+    }
+    catch (const std::exception &err)
+    {
+        CPLError(CE_Failure, CPLE_AppDefined, "Unexpected exception: %s",
+                 err.what());
+        return std::string();
+    }
+}
+
 /************************************************************************/
 /*                             GDALFootprint()                          */
 /************************************************************************/
@@ -1189,77 +1369,32 @@ GDALFootprintOptionsNew(char **papszArgv,
 {
     auto psOptions = std::make_unique<GDALFootprintOptions>();
 
-    bool bGotSourceFilename = false;
-    bool bGotDestFilename = false;
     /* -------------------------------------------------------------------- */
-    /*      Handle command line arguments.                                  */
+    /*      Parse arguments.                                                */
     /* -------------------------------------------------------------------- */
-    const int argc = CSLCount(papszArgv);
-    for (int i = 0; papszArgv != nullptr && i < argc; i++)
+
+    CPLStringList aosArgv;
+
+    if (papszArgv)
     {
-        if (i < argc - 1 &&
-            (EQUAL(papszArgv[i], "-of") || EQUAL(papszArgv[i], "-f")))
+        const int nArgc = CSLCount(papszArgv);
+        for (int i = 0; i < nArgc; i++)
         {
-            ++i;
-            psOptions->osFormat = papszArgv[i];
-            psOptions->bCreateOutput = true;
+            aosArgv.AddString(papszArgv[i]);
         }
+    }
 
-        else if (EQUAL(papszArgv[i], "-q") || EQUAL(papszArgv[i], "-quiet"))
-        {
-            if (psOptionsForBinary)
-            {
-                psOptionsForBinary->bQuiet = true;
-            }
-            else
-            {
-                CPLError(CE_Failure, CPLE_NotSupported,
-                         "%s switch only supported from gdal_footprint binary.",
-                         papszArgv[i]);
-                return nullptr;
-            }
-        }
+    try
+    {
+        auto argParser = GDALFootprintAppOptionsGetParser(psOptions.get(),
+                                                          psOptionsForBinary);
 
-        else if (i < argc - 1 && EQUAL(papszArgv[i], "-oo"))
-        {
-            i++;
-            if (psOptionsForBinary)
-            {
-                psOptionsForBinary->aosOpenOptions.AddString(papszArgv[i]);
-            }
-            else
-            {
-                CPLError(
-                    CE_Failure, CPLE_NotSupported,
-                    "-oo switch only supported from gdal_footprint binary.");
-            }
-        }
+        argParser->parse_args_without_binary_name(aosArgv.List());
 
-        else if (i < argc - 1 && EQUAL(papszArgv[i], "-t_cs"))
+        if (argParser->is_used("-t_srs"))
         {
-            i++;
-            const std::string osVal(papszArgv[i]);
-            if (osVal == "georef")
-            {
-                psOptions->bOutCSGeoref = true;
-                psOptions->bOutCSGeorefRequested = true;
-            }
-            else if (osVal == "pixel")
-            {
-                psOptions->bOutCSGeoref = false;
-            }
-            else
-            {
-                CPLError(CE_Failure, CPLE_NotSupported,
-                         "Invalid value for -t_cs");
-                return nullptr;
-            }
-        }
 
-        else if (i < argc - 1 && EQUAL(papszArgv[i], "-t_srs"))
-        {
-            i++;
-            const std::string osVal(papszArgv[i]);
+            const std::string osVal(argParser->get<std::string>("-t_srs"));
             if (psOptions->oOutputSRS.SetFromUserInput(osVal.c_str()) !=
                 OGRERR_NONE)
             {
@@ -1271,83 +1406,17 @@ GDALFootprintOptionsNew(char **papszArgv,
                 OAMS_TRADITIONAL_GIS_ORDER);
         }
 
-        else if (i < argc - 1 && EQUAL(papszArgv[i], "-b"))
+        if (argParser->is_used("-max_points"))
         {
-            i++;
-            psOptions->anBands.push_back(atoi(papszArgv[i]));
-        }
-
-        else if (i < argc - 1 && EQUAL(papszArgv[i], "-combine_bands"))
-        {
-            i++;
-            if (EQUAL(papszArgv[i], "union"))
-                psOptions->bCombineBandsUnion = true;
-            else if (EQUAL(papszArgv[i], "intersection"))
-                psOptions->bCombineBandsUnion = false;
-            else
-            {
-                CPLError(CE_Failure, CPLE_NotSupported,
-                         "Invalid value for -combine_bands");
-                return nullptr;
-            }
-        }
-
-        else if (i < argc - 1 && EQUAL(papszArgv[i], "-srcnodata"))
-        {
-            i++;
-            psOptions->osSrcNoData = papszArgv[i];
-        }
-
-        else if (i < argc - 1 && EQUAL(papszArgv[i], "-lco"))
-        {
-            i++;
-            psOptions->aosLCO.AddString(papszArgv[i]);
-        }
-
-        else if (i < argc - 1 && EQUAL(papszArgv[i], "-dsco"))
-        {
-            i++;
-            psOptions->aosDSCO.AddString(papszArgv[i]);
-        }
-
-        else if (i < argc - 1 && EQUAL(papszArgv[i], "-lyr_name"))
-        {
-            i++;
-            psOptions->osDestLayerName = papszArgv[i];
-        }
-
-        else if (EQUAL(papszArgv[i], "-split_polys"))
-        {
-            psOptions->bSplitPolys = true;
-        }
-
-        else if (EQUAL(papszArgv[i], "-convex_hull"))
-        {
-            psOptions->bConvexHull = true;
-        }
-
-        else if (i < argc - 1 && EQUAL(papszArgv[i], "-densify"))
-        {
-            i++;
-            psOptions->dfDensifyDistance = CPLAtof(papszArgv[i]);
-        }
-
-        else if (i < argc - 1 && EQUAL(papszArgv[i], "-simplify"))
-        {
-            i++;
-            psOptions->dfSimplifyTolerance = CPLAtof(papszArgv[i]);
-        }
-
-        else if (i < argc - 1 && EQUAL(papszArgv[i], "-max_points"))
-        {
-            i++;
-            if (EQUAL(papszArgv[i], "unlimited"))
+            const std::string maxPoints{
+                argParser->get<std::string>("-max_points")};
+            if (maxPoints == "unlimited")
             {
                 psOptions->nMaxPoints = 0;
             }
             else
             {
-                psOptions->nMaxPoints = atoi(papszArgv[i]);
+                psOptions->nMaxPoints = atoi(maxPoints.c_str());
                 if (psOptions->nMaxPoints > 0 && psOptions->nMaxPoints < 3)
                 {
                     CPLError(CE_Failure, CPLE_NotSupported,
@@ -1357,83 +1426,13 @@ GDALFootprintOptionsNew(char **papszArgv,
             }
         }
 
-        else if (i < argc - 1 && EQUAL(papszArgv[i], "-min_ring_area"))
-        {
-            i++;
-            psOptions->dfMinRingArea = CPLAtof(papszArgv[i]);
-        }
-
-        else if (EQUAL(papszArgv[i], "-overwrite"))
-        {
-            if (psOptionsForBinary)
-            {
-                psOptionsForBinary->bOverwrite = true;
-            }
-            else
-            {
-                CPLError(CE_Failure, CPLE_NotSupported,
-                         "-overwrite switch only supported from gdal_footprint "
-                         "binary.");
-                return nullptr;
-            }
-        }
-
-        else if (i < argc - 1 && EQUAL(papszArgv[i], "-location_field_name"))
-        {
-            i++;
-            psOptions->osLocationFieldName = papszArgv[i];
-        }
-
-        else if (EQUAL(papszArgv[i], "-no_location"))
-        {
-            psOptions->osLocationFieldName.clear();
-        }
-
-        else if (EQUAL(papszArgv[i], "-write_absolute_path"))
-        {
-            psOptions->bAbsolutePath = true;
-        }
-
-        else if (i < argc - 1 && EQUAL(papszArgv[i], "-ovr"))
-        {
-            i++;
-            psOptions->nOvrIndex = atoi(papszArgv[i]);
-        }
-
-        else if (papszArgv[i][0] == '-')
-        {
-            CPLError(CE_Failure, CPLE_NotSupported, "Unknown option name '%s'",
-                     papszArgv[i]);
-            return nullptr;
-        }
-        else if (!bGotSourceFilename)
-        {
-            bGotSourceFilename = true;
-            if (psOptionsForBinary)
-            {
-                psOptionsForBinary->osSource = papszArgv[i];
-            }
-            else
-            {
-                CPLError(CE_Failure, CPLE_NotSupported,
-                         "{source_filename} only supported from gdal_footprint "
-                         "binary.");
-                return nullptr;
-            }
-        }
-        else if (!bGotDestFilename)
-        {
-            bGotDestFilename = true;
-            CPLAssert(psOptionsForBinary);
-            psOptionsForBinary->bDestSpecified = true;
-            psOptionsForBinary->osDest = papszArgv[i];
-        }
-        else
-        {
-            CPLError(CE_Failure, CPLE_NotSupported,
-                     "Too many command options '%s'", papszArgv[i]);
-            return nullptr;
-        }
+        psOptions->bCreateOutput = !psOptions->osFormat.empty();
+    }
+    catch (const std::exception &err)
+    {
+        CPLError(CE_Failure, CPLE_AppDefined, "Unexpected exception: %s",
+                 err.what());
+        return nullptr;
     }
 
     if (!psOptions->bOutCSGeoref && !psOptions->oOutputSRS.IsEmpty())
@@ -1443,11 +1442,9 @@ GDALFootprintOptionsNew(char **papszArgv,
         return nullptr;
     }
 
-    if (!psOptions->osSrcNoData.empty() && psOptions->nOvrIndex >= 0)
+    if (psOptions->bClearLocation)
     {
-        CPLError(CE_Failure, CPLE_AppDefined,
-                 "-srcnodata and -ovr are mutually exclusive.");
-        return nullptr;
+        psOptions->osLocationFieldName.clear();
     }
 
     if (psOptionsForBinary)
