@@ -8,23 +8,7 @@
  * Copyright (c) 2004, Pirmin Kalberer, Sourcepole AG
  * Copyright (c) 2008-2013, Even Rouault <even dot rouault at spatialys.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_conv.h"
@@ -171,259 +155,6 @@ GIntBig OGRILI1Layer::GetFeatureCount(int bForce)
     }
 }
 
-#ifndef d2str_defined
-#define d2str_defined
-
-static const char *d2str(double val)
-{
-    if (val == (int)val)
-        return CPLSPrintf("%d", (int)val);
-    if (fabs(val) < 370)
-        return CPLSPrintf("%.16g", val);
-    if (fabs(val) > 100000000.0)
-        return CPLSPrintf("%.16g", val);
-
-    return CPLSPrintf("%.3f", val);
-}
-#endif
-
-static void AppendCoordinateList(OGRLineString *poLine, OGRILI1DataSource *poDS)
-{
-    const bool b3D = CPL_TO_BOOL(wkbHasZ(poLine->getGeometryType()));
-
-    for (int iPoint = 0; iPoint < poLine->getNumPoints(); iPoint++)
-    {
-        if (iPoint == 0)
-            VSIFPrintfL(poDS->GetTransferFile(), "STPT");
-        else
-            VSIFPrintfL(poDS->GetTransferFile(), "LIPT");
-        VSIFPrintfL(poDS->GetTransferFile(), " %s",
-                    d2str(poLine->getX(iPoint)));
-        VSIFPrintfL(poDS->GetTransferFile(), " %s",
-                    d2str(poLine->getY(iPoint)));
-        if (b3D)
-            VSIFPrintfL(poDS->GetTransferFile(), " %s",
-                        d2str(poLine->getZ(iPoint)));
-        VSIFPrintfL(poDS->GetTransferFile(), "\n");
-    }
-    VSIFPrintfL(poDS->GetTransferFile(), "ELIN\n");
-}
-
-static void AppendCompoundCurve(OGRCompoundCurve *poCC, OGRILI1DataSource *poDS)
-{
-    for (int iMember = 0; iMember < poCC->getNumCurves(); iMember++)
-    {
-        OGRCurve *poGeometry = poCC->getCurve(iMember);
-        int b3D = wkbHasZ(poGeometry->getGeometryType());
-        int bIsArc = (poGeometry->getGeometryType() == wkbCircularString ||
-                      poGeometry->getGeometryType() == wkbCircularStringZ);
-        OGRSimpleCurve *poLine = (OGRSimpleCurve *)poGeometry;
-        for (int iPoint = 0; iPoint < poLine->getNumPoints(); iPoint++)
-        {
-            // Skip last point in curve member
-            if (iPoint == poLine->getNumPoints() - 1 &&
-                iMember < poCC->getNumCurves() - 1)
-                continue;
-            if (iMember == 0 && iPoint == 0)
-                VSIFPrintfL(poDS->GetTransferFile(), "STPT");
-            else if (bIsArc && iPoint == 1)
-                VSIFPrintfL(poDS->GetTransferFile(), "ARCP");
-            else
-                VSIFPrintfL(poDS->GetTransferFile(), "LIPT");
-            VSIFPrintfL(poDS->GetTransferFile(), " %s",
-                        d2str(poLine->getX(iPoint)));
-            VSIFPrintfL(poDS->GetTransferFile(), " %s",
-                        d2str(poLine->getY(iPoint)));
-            if (b3D)
-                VSIFPrintfL(poDS->GetTransferFile(), " %s",
-                            d2str(poLine->getZ(iPoint)));
-            VSIFPrintfL(poDS->GetTransferFile(), "\n");
-        }
-    }
-    VSIFPrintfL(poDS->GetTransferFile(), "ELIN\n");
-}
-
-int OGRILI1Layer::GeometryAppend(OGRGeometry *poGeometry)
-{
-#ifdef DEBUG_VERBOSE
-    CPLDebug("OGR_ILI", "OGRILI1Layer::GeometryAppend OGRGeometryType: %s",
-             OGRGeometryTypeToName(poGeometry->getGeometryType()));
-#endif
-    /* -------------------------------------------------------------------- */
-    /*      2D Point                                                        */
-    /* -------------------------------------------------------------------- */
-    if (poGeometry->getGeometryType() == wkbPoint)
-    {
-        /* embedded in from non-geometry fields */
-    }
-    /* -------------------------------------------------------------------- */
-    /*      3D Point                                                        */
-    /* -------------------------------------------------------------------- */
-    else if (poGeometry->getGeometryType() == wkbPoint25D)
-    {
-        /* embedded in from non-geometry fields */
-    }
-
-    /* -------------------------------------------------------------------- */
-    /*      LineString and LinearRing                                       */
-    /* -------------------------------------------------------------------- */
-    else if (poGeometry->getGeometryType() == wkbLineString ||
-             poGeometry->getGeometryType() == wkbLineString25D)
-    {
-        AppendCoordinateList(poGeometry->toLineString(), poDS);
-    }
-
-    /* -------------------------------------------------------------------- */
-    /*      Polygon                                                         */
-    /* -------------------------------------------------------------------- */
-    else if (poGeometry->getGeometryType() == wkbPolygon ||
-             poGeometry->getGeometryType() == wkbPolygon25D)
-    {
-        OGRPolygon *poPolygon = poGeometry->toPolygon();
-        for (auto &&poRing : poPolygon)
-        {
-            if (!GeometryAppend(poRing))
-                return FALSE;
-        }
-    }
-
-    /* -------------------------------------------------------------------- */
-    /*      MultiPolygon                                                    */
-    /* -------------------------------------------------------------------- */
-    else if (wkbFlatten(poGeometry->getGeometryType()) == wkbMultiPolygon ||
-             wkbFlatten(poGeometry->getGeometryType()) == wkbMultiLineString ||
-             wkbFlatten(poGeometry->getGeometryType()) == wkbMultiPoint ||
-             wkbFlatten(poGeometry->getGeometryType()) ==
-                 wkbGeometryCollection ||
-             wkbFlatten(poGeometry->getGeometryType()) == wkbMultiCurve ||
-             wkbFlatten(poGeometry->getGeometryType()) == wkbMultiCurveZ)
-    {
-        OGRGeometryCollection *poGC = poGeometry->toGeometryCollection();
-
-#if 0
-        // TODO: Why this large NOP block?
-        if( wkbFlatten(poGeometry->getGeometryType()) == wkbMultiPolygon )
-        {
-        }
-        else if( wkbFlatten(poGeometry->getGeometryType()) == wkbMultiLineString )
-        {
-        }
-        else if( wkbFlatten(poGeometry->getGeometryType()) == wkbMultiPoint )
-        {
-        }
-        else
-        {
-        }
-#endif
-
-        for (auto &&poMember : poGC)
-        {
-            if (!GeometryAppend(poMember))
-                return FALSE;
-        }
-    }
-    else if (poGeometry->getGeometryType() == wkbCompoundCurve ||
-             poGeometry->getGeometryType() == wkbCompoundCurveZ)
-    {
-        AppendCompoundCurve(poGeometry->toCompoundCurve(), poDS);
-    }
-    else
-    {
-        CPLError(CE_Warning, CPLE_AppDefined,
-                 "Skipping unknown geometry type '%s'",
-                 OGRGeometryTypeToName(poGeometry->getGeometryType()));
-        return FALSE;
-    }
-
-    return TRUE;
-}
-
-/************************************************************************/
-/*                           ICreateFeature()                            */
-/************************************************************************/
-
-OGRErr OGRILI1Layer::ICreateFeature(OGRFeature *poFeature)
-{
-    // FIXME: tid not thread safe
-    static long tid = -1;  // system generated TID (must be unique within table)
-    VSILFILE *fp = poDS->GetTransferFile();
-    VSIFPrintfL(fp, "OBJE");
-
-    if (poFeatureDefn->GetFieldCount() &&
-        !EQUAL(poFeatureDefn->GetFieldDefn(0)->GetNameRef(), "TID"))
-    {
-        // Input is not generated from an Interlis 1 source
-        if (poFeature->GetFID() != OGRNullFID)
-            tid = (int)poFeature->GetFID();
-        else
-            ++tid;
-        VSIFPrintfL(fp, " %ld", tid);
-        // Embedded geometry
-        if (poFeature->GetGeometryRef() != nullptr)
-        {
-            OGRGeometry *poGeometry = poFeature->GetGeometryRef();
-            // 2D Point
-            if (poGeometry->getGeometryType() == wkbPoint)
-            {
-                OGRPoint *poPoint = poGeometry->toPoint();
-
-                VSIFPrintfL(fp, " %s", d2str(poPoint->getX()));
-                VSIFPrintfL(fp, " %s", d2str(poPoint->getY()));
-            }
-            // 3D Point
-            else if (poGeometry->getGeometryType() == wkbPoint25D)
-            {
-                OGRPoint *poPoint = poGeometry->toPoint();
-
-                VSIFPrintfL(fp, " %s", d2str(poPoint->getX()));
-                VSIFPrintfL(fp, " %s", d2str(poPoint->getY()));
-                VSIFPrintfL(fp, " %s", d2str(poPoint->getZ()));
-            }
-        }
-    }
-
-    // Write all fields.
-    for (int iField = 0; iField < poFeatureDefn->GetFieldCount(); iField++)
-    {
-        if (poFeature->IsFieldSetAndNotNull(iField))
-        {
-            const char *pszRaw = poFeature->GetFieldAsString(iField);
-            if (poFeatureDefn->GetFieldDefn(iField)->GetType() == OFTString)
-            {
-                // Interlis 1 encoding is ISO 8859-1 (Latin1) -> Recode from
-                // UTF-8
-                char *pszString =
-                    CPLRecode(pszRaw, CPL_ENC_UTF8, CPL_ENC_ISO8859_1);
-                // Replace spaces
-                for (size_t i = 0; pszString[i] != '\0'; i++)
-                {
-                    if (pszString[i] == ' ')
-                        pszString[i] = '_';
-                }
-                VSIFPrintfL(fp, " %s", pszString);
-                CPLFree(pszString);
-            }
-            else
-            {
-                VSIFPrintfL(fp, " %s", pszRaw);
-            }
-        }
-        else
-        {
-            VSIFPrintfL(fp, " @");
-        }
-    }
-    VSIFPrintfL(fp, "\n");
-
-    // Write out Geometry
-    if (poFeature->GetGeometryRef() != nullptr)
-    {
-        GeometryAppend(poFeature->GetGeometryRef());
-    }
-
-    return OGRERR_NONE;
-}
-
 /************************************************************************/
 /*                           TestCapability()                           */
 /************************************************************************/
@@ -435,24 +166,9 @@ int OGRILI1Layer::TestCapability(CPL_UNUSED const char *pszCap)
     if (EQUAL(pszCap, OLCZGeometries))
         return TRUE;
 
-    if (EQUAL(pszCap, OLCCreateField) || EQUAL(pszCap, OLCSequentialWrite))
-        return poDS->GetTransferFile() != nullptr;
-
     return FALSE;
 }
 
-/************************************************************************/
-/*                            CreateField()                             */
-/************************************************************************/
-
-OGRErr OGRILI1Layer::CreateField(const OGRFieldDefn *poField,
-                                 int /* bApproxOK */)
-{
-    poFeatureDefn->AddFieldDefn(poField);
-
-    return OGRERR_NONE;
-}
-
 /************************************************************************/
 /*                         Internal routines                            */
 /************************************************************************/
@@ -460,12 +176,9 @@ OGRErr OGRILI1Layer::CreateField(const OGRFieldDefn *poField,
 void OGRILI1Layer::JoinGeomLayers()
 {
     bGeomsJoined = true;
-    bool bResetConfigOption = false;
-    if (EQUAL(CPLGetConfigOption("OGR_ARC_STEPSIZE", ""), ""))
-    {
-        bResetConfigOption = true;
-        CPLSetThreadLocalConfigOption("OGR_ARC_STEPSIZE", "0.96");
-    }
+
+    CPLConfigOptionSetter oSetter("OGR_ARC_STEPSIZE", "0.96",
+                                  /* bSetOnlyIfUndefined = */ true);
 
     for (GeomFieldInfos::const_iterator it = oGeomFieldInfos.begin();
          it != oGeomFieldInfos.end(); ++it)
@@ -493,9 +206,6 @@ void OGRILI1Layer::JoinGeomLayers()
             }
         }
     }
-
-    if (bResetConfigOption)
-        CPLSetThreadLocalConfigOption("OGR_ARC_STEPSIZE", nullptr);
 }
 
 void OGRILI1Layer::JoinSurfaceLayer(OGRILI1Layer *poSurfaceLineLayer,
